@@ -32,8 +32,15 @@ from .base.address import GOVERNANCE_SCORE_ADDRESS
 from .base.address import SYSTEM_SCORE_ADDRESS
 from .base.block import Block
 from .base.exception import (
-    ExceptionCode, IconServiceBaseException, IconScoreException, InvalidBaseTransactionException,
-    InternalServiceErrorException, DatabaseException)
+    ExceptionCode,
+    IconServiceBaseException,
+    IconScoreException,
+    InvalidBaseTransactionException,
+    InternalServiceErrorException,
+    DatabaseException,
+    InvalidBalanceException,
+    InvalidParamsException,
+)
 from .base.message import Message
 from .base.transaction import Transaction
 from .base.type_converter_templates import ConstantKeys
@@ -47,7 +54,8 @@ from .icon_constant import (
     ICON_DEX_DB_NAME, IconServiceFlag, ConfigKey,
     Revision, BASE_TRANSACTION_INDEX,
     IISS_DB, STEP_LOG_TAG, BlockVoteStatus, WAL_LOG_TAG, ROLLBACK_LOG_TAG,
-    BLOCK_INVOKE_TIMEOUT_S, RevisionChangedFlag, RPCMethod
+    BLOCK_INVOKE_TIMEOUT_S, RevisionChangedFlag, RPCMethod,
+    DataType
 )
 from .iconscore.context.context import ContextContainer
 from .iconscore.icon_pre_validator import IconPreValidator
@@ -64,6 +72,9 @@ from .iconscore.icon_score_trace import Trace, TraceType
 from .icx import IcxEngine, IcxStorage
 from .icx.issue import IssueEngine, IssueStorage
 from .icx.issue.base_transaction_creator import BaseTransactionCreator
+from .icx.storage import AccountPartFlag
+from .icx.unstake_patcher import INVALID_EXPIRED_UNSTAKES_FILENAME
+from .icx.unstake_patcher import UnstakePatcher
 from .iiss import check_decentralization_condition
 from .iiss.engine import Engine as IISSEngine
 from .iiss.reward_calc import RewardCalcStorage
@@ -82,7 +93,6 @@ from .utils import sha3_256, int_to_bytes, ContextEngine, ContextStorage
 from .utils import to_camel_case, bytes_to_hex
 from .utils.bloom import BloomFilter
 from .utils.timer import Timer
-from .utils.locked import is_address_locked
 
 if TYPE_CHECKING:
     from .iconscore.icon_score_event_log import EventLog
@@ -116,6 +126,7 @@ class IconServiceEngine(ContextContainer):
         self._backup_cleaner: Optional[BackupCleaner] = None
         self._conf: Optional[Dict[str, Union[str, int]]] = None
         self._block_invoke_timeout_s: int = BLOCK_INVOKE_TIMEOUT_S
+        self._log_dir: str = "."
 
         # JSON-RPC handlers
         self._handlers = {
@@ -125,7 +136,8 @@ class IconServiceEngine(ContextContainer):
             RPCMethod.ISE_GET_STATUS: self._handle_ise_get_status,
             RPCMethod.ICX_CALL: self._handle_icx_call,
             RPCMethod.DEBUG_ESTIMATE_STEP: self._handle_estimate_step,
-            RPCMethod.ICX_SEND_TRANSACTION: self._handle_icx_send_transaction
+            RPCMethod.ICX_SEND_TRANSACTION: self._handle_icx_send_transaction,
+            RPCMethod.DEBUG_GET_ACCOUNT: self._handle_debug_get_account
         }
 
         self._precommit_data_manager = PrecommitDataManager()
@@ -218,11 +230,10 @@ class IconServiceEngine(ContextContainer):
 
         self._set_block_invoke_timeout(conf)
 
-        self._set_block_invoke_timeout(conf)
-
         # DO NOT change the values in conf
         self._conf = conf
         self._precommit_data_writer = PrecommitDataWriter(log_dir)
+        self._log_dir = log_dir
 
     def _init_component_context(self):
         engine: 'ContextEngine' = ContextEngine(deploy=DeployEngine(),
@@ -449,7 +460,8 @@ class IconServiceEngine(ContextContainer):
             tx_timer.start()
 
             for index, tx_request in enumerate(tx_requests):
-                Logger.debug(_TAG, f"INVOKE tx: {tx_request}")
+                tx_hash: Optional[bytes] = tx_request["params"].get("txHash")
+                Logger.debug(_TAG, f"INVOKE tx: tx_hash={bytes_to_hex(tx_hash)} {tx_request}")
 
                 # Adjust the number of transactions in a block to make sure that
                 # a leader can broadcast a block candidate to validators in a specific period.
@@ -480,6 +492,9 @@ class IconServiceEngine(ContextContainer):
                 if context.revision >= Revision.IISS.value:
                     context.block_batch.block.cumulative_fee += tx_result.step_price * tx_result.step_used
 
+                if context.is_revision_changed(Revision.FIX_BALANCE_BUG.value):
+                    self._run_unstake_patcher(context)
+
                 Logger.debug(_TAG, f"INVOKE txResult: {tx_result}")
 
         if self._check_end_block_height_of_calc(context):
@@ -777,7 +792,7 @@ class IconServiceEngine(ContextContainer):
             return
 
         dirty_prep: 'PRep' = context.get_prep(prev_block_generator, mutable=True)
-        assert isinstance(dirty_prep, PRep), f"dirty_prep: {address}"
+        assert isinstance(dirty_prep, PRep), f"dirty_prep: {dirty_prep.address}"
 
         dirty_prep.last_generate_block_height = context.block.height - 1
         context.put_dirty_prep(dirty_prep)
@@ -1056,7 +1071,7 @@ class IconServiceEngine(ContextContainer):
         ret = self._call(context, method, params)
         return ret
 
-    def validate_transaction(self, request: dict) -> None:
+    def validate_transaction(self, request: dict, origin_request: dict) -> None:
         """Validate JSON-RPC transaction request
         before putting it into transaction pool
 
@@ -1064,6 +1079,7 @@ class IconServiceEngine(ContextContainer):
         on JSON-RPC Server
         IconPreValidator focuses on business logic and semantic problems
 
+        :param origin_request: JSON_RPC Original request for more strict validate
         :param request: JSON-RPC request
             values in request have already been converted to original format
             in IconInnerService
@@ -1081,6 +1097,8 @@ class IconServiceEngine(ContextContainer):
         context = self._context_factory.create(IconScoreContextType.QUERY, self._get_last_block())
         context.set_step_counter()
 
+        origin_params = origin_request['params']
+
         try:
             self._push_context(context)
 
@@ -1093,12 +1111,13 @@ class IconServiceEngine(ContextContainer):
                 data = params['data']
                 input_size = get_input_data_size(context.revision, data)
                 minimum_step += input_size * context.inv_container.step_costs.get(StepType.INPUT, 0)
-
+            self._icon_pre_validator.origin_request_execute(origin_params, context.revision)
             self._icon_pre_validator.execute(context, params, step_price, minimum_step)
 
-            # SCORE updating is not blocked by SCORE blacklist
-            if 'dataType' in params and params['dataType'] == 'call':
+            if to.is_contract:
+                # SCORE updating is not blocked by SCORE blacklist
                 IconScoreContextUtil.validate_score_blacklist(context, to)
+
         finally:
             self._pop_context()
 
@@ -1173,12 +1192,17 @@ class IconServiceEngine(ContextContainer):
         :return:
         """
 
-        icon_score_address: Address = params['to']
+        to: Address = params['to']
         data_type = params.get('dataType', None)
         data = params.get('data', None)
 
         context.step_counter.apply_step(StepType.CONTRACT_CALL, 1)
-        return IconScoreEngine.query(context, icon_score_address, data_type, data)
+        if to.is_contract:
+            return IconScoreEngine.query(context, to, data_type, data)
+
+        raise InvalidParamsException(
+            f"Mismatch between to and dataType: to={to} dataType={data_type}"
+        )
 
     def _handle_icx_send_transaction(self,
                                      context: 'IconScoreContext',
@@ -1200,6 +1224,11 @@ class IconServiceEngine(ContextContainer):
             # process the transaction
             self._process_transaction(context, params, tx_result)
             tx_result.status = TransactionResult.SUCCESS
+        except InvalidBalanceException as e:
+            # If InvalidBalanceException is raised, stop to invoke this block
+            # Because balance integrity is broken
+            Logger.exception(tag=_TAG, msg=str(e))
+            raise e
         except BaseException as e:
             tx_result.failure = self._get_failure_from_exception(e)
             trace = self._get_trace_from_exception(context.current_address, e)
@@ -1270,15 +1299,30 @@ class IconServiceEngine(ContextContainer):
         context.step_counter.apply_step(StepType.INPUT, input_size)
 
         to: Address = params['to']
-        data_type: str = params.get('dataType')
+        data_type: Optional[str] = params.get('dataType')
+
+        self._validate_data_type(context, to, data_type)
 
         # Can't transfer ICX to system SCORE
-        if data_type in (None, 'call', 'message') and to != SYSTEM_SCORE_ADDRESS:
+        if data_type in (None, DataType.CALL, DataType.MESSAGE):
             self._transfer_coin(context, params)
 
         if to.is_contract:
             tx_result.score_address = self._handle_score_invoke(context, to, params)
 
+    def _validate_data_type(
+            self, context: 'IconScoreContext', to: 'Address', data_type: Optional[str]
+    ):
+        try:
+            self._icon_pre_validator.validate_data_type(context, to, data_type)
+        except InvalidParamsException as e:
+            if data_type == DataType.CALL:
+                context.step_counter.apply_step(StepType.CONTRACT_CALL, 1)
+            elif data_type == DataType.DEPLOY:
+                context.step_counter.apply_step(StepType.CONTRACT_UPDATE, 1)
+
+            raise e
+
     @classmethod
     def _transfer_coin(cls,
                        context: 'IconScoreContext',
@@ -1294,6 +1338,14 @@ class IconServiceEngine(ContextContainer):
         to: 'Address' = params['to']
         value: int = params.get('value', 0)
 
+        if (
+                to == SYSTEM_SCORE_ADDRESS
+                and context.revision < Revision.BURN_V2_ENABLED.value
+        ):
+            # Prevent system score from receiving ICX
+            # if context.revision is less than Revision.BURN_V2_ENABLED(12)
+            return
+
         context.engine.icx.transfer(context, from_, to, value)
 
     @classmethod
@@ -1361,9 +1413,9 @@ class IconServiceEngine(ContextContainer):
         data_type: str = params.get('dataType')
         data: dict = params.get('data')
 
-        if data_type == 'deploy':
+        if data_type == DataType.DEPLOY:
             return context.engine.deploy.invoke(context, to, data)
-        elif data_type == 'deposit':
+        elif data_type == DataType.DEPOSIT:
             self._deposit_handler.handle_deposit_request(context, data)
             return None
         else:
@@ -1380,9 +1432,10 @@ class IconServiceEngine(ContextContainer):
         # exceptions for mainnet backward compatibility:
         #   - dataType is not 'call'
         if to == SYSTEM_SCORE_ADDRESS:
-            if context.revision < Revision.SYSTEM_SCORE_ENABLED.value and data_type != 'call':
-                return True
-            return False
+            return (
+                context.revision < Revision.SYSTEM_SCORE_ENABLED.value
+                and data_type != DataType.CALL
+            )
 
         return True
 
@@ -2099,3 +2152,52 @@ class IconServiceEngine(ContextContainer):
                 return False
 
         return True
+
+    def _run_unstake_patcher(self, context: 'IconScoreContext'):
+        Logger.info(tag=_TAG, msg="_run_unstake_patcher() start")
+
+        if context.revision == Revision.FIX_BALANCE_BUG.value:
+            try:
+                path: Optional[str] = self._conf.get(
+                    ConfigKey.INVALID_EXPIRED_UNSTAKES_PATH, None)
+                Logger.info(tag="UNSTAKE", msg=f"path: {path}")
+
+                patcher = UnstakePatcher.from_path(path)
+                patcher.run(context)
+
+                names: List[str] = INVALID_EXPIRED_UNSTAKES_FILENAME.split(".")
+                report_path: str = os.path.join(self._log_dir, f"{names[0]}_report.json")
+                patcher.write_result(report_path)
+            except BaseException as e:
+                Logger.exception(tag=_TAG, msg=f"Failed to run UnstakePatcher: {str(e)}")
+        else:
+            Logger.exception(
+                tag=_TAG,
+                msg="Failed to run unstake_patcher: "
+                    f"revision({context.revision}) != {Revision.FIX_BALANCE_BUG.value}"
+            )
+
+        Logger.info(tag=_TAG, msg="_run_unstake_patcher() end")
+
+    @classmethod
+    def _handle_debug_get_account(
+            cls,
+            context: 'IconScoreContext',
+            params: dict
+    ) -> dict:
+        """
+        Handles get raw data of account in StateDB
+
+        :param context: context
+        :param params: parameters of account address and filter
+        :return: account raw data of stateDB
+        """
+
+        address: 'Address' = params["address"]
+        account_filter: 'AccountPartFlag' = AccountPartFlag(params["filter"])
+        raw_data: dict = context.engine.icx.get_account_raw_data(
+            context=context,
+            address=address,
+            account_filter=account_filter
+        )
+        return raw_data
