@@ -14,15 +14,19 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import TYPE_CHECKING, Any
+from typing import TYPE_CHECKING, Any, Optional
 
 from iconcommons.logger import Logger
 
-from .icon_score_context import IconScoreContextType
 from .icon_score_step import get_input_data_size
-from ..base.address import Address, SYSTEM_SCORE_ADDRESS, generate_score_address
+
+from ..base.address import Address, AddressPrefix, SYSTEM_SCORE_ADDRESS, generate_score_address, is_icon_address_valid
 from ..base.exception import InvalidRequestException, InvalidParamsException, OutOfBalanceException
-from ..icon_constant import FIXED_FEE, MAX_DATA_SIZE, DEFAULT_BYTE_SIZE, DATA_BYTE_ORDER, Revision, DeployState
+from ..base.type_converter_templates import ConstantKeys
+from ..icon_constant import (
+    FIXED_FEE, MAX_DATA_SIZE, DEFAULT_BYTE_SIZE,
+    DATA_BYTE_ORDER, Revision, DeployState, DataType
+)
 from ..utils import is_lowercase_hex_string
 from ..utils.locked import is_address_locked
 
@@ -32,6 +36,43 @@ if TYPE_CHECKING:
 
 
 TAG = "PV"
+REQUEST_PARAMS = (
+    ConstantKeys.VERSION,
+    ConstantKeys.STEP_LIMIT,
+    ConstantKeys.NID,
+    ConstantKeys.TIMESTAMP,
+    ConstantKeys.VALUE,
+    ConstantKeys.NONCE,
+    ConstantKeys.FROM,
+    ConstantKeys.TO,
+    ConstantKeys.SIGNATURE,
+    ConstantKeys.DATA_TYPE,
+    ConstantKeys.DATA,
+)
+
+REQUIRED_PARAMS = (
+    ConstantKeys.VERSION,
+    ConstantKeys.STEP_LIMIT,
+    ConstantKeys.NID,
+    ConstantKeys.TIMESTAMP,
+    ConstantKeys.FROM,
+    ConstantKeys.TO,
+    ConstantKeys.SIGNATURE
+)
+
+INT_PARAMS = (
+    ConstantKeys.VERSION,
+    ConstantKeys.STEP_LIMIT,
+    ConstantKeys.NID,
+    ConstantKeys.TIMESTAMP,
+    ConstantKeys.VALUE,
+    ConstantKeys.NONCE
+)
+
+ADDR_PARAMS = (
+    ConstantKeys.FROM,
+    ConstantKeys.TO
+)
 
 
 class IconPreValidator:
@@ -45,6 +86,69 @@ class IconPreValidator:
         """
         pass
 
+    def origin_request_execute(self, params: dict, revision: int):
+        if revision < Revision.IMPROVED_PRE_VALIDATOR.value:
+            return
+        self.origin_pre_validate_version(params)
+        self.origin_pre_validate_params(params)
+        self.origin_validate_fields(params)
+
+    def origin_validate_fields(self, params: dict):
+        for param, value in params.items():
+            self.origin_validate_param(param)
+            self.origin_validate_value(param, value)
+
+    @classmethod
+    def origin_pre_validate_version(cls, params: dict):
+        version: str = params.get(ConstantKeys.VERSION, None)
+        if version != '0x3':
+            raise InvalidRequestException(f'Invalid message version, got {version}')
+
+    @classmethod
+    def origin_pre_validate_params(cls, params: dict):
+        if len(params) > len(REQUEST_PARAMS):
+            raise InvalidRequestException('Unexpected Parameters')
+
+        required_results = [
+            required_key
+            for required_key
+            in REQUIRED_PARAMS
+            if required_key not in params
+        ]
+
+        if required_results:
+            raise InvalidRequestException(
+                f'Not included required parameters, missing parameters {required_results}'
+            )
+
+    @classmethod
+    def origin_validate_param(cls, param: str):
+        if param not in REQUEST_PARAMS:
+            raise InvalidParamsException(f'Unexpected Parameters, got {param}')
+
+    @classmethod
+    def origin_validate_value(cls, param: str, value: str):
+        if param in INT_PARAMS:
+            if not cls.is_integer_type(value):
+                raise InvalidRequestException(f'Unexpected INT Type, got {value}')
+        elif param in ADDR_PARAMS:
+            if not cls.is_address_type(value):
+                raise InvalidRequestException(f'Unexpected Address Type, got {value}')
+
+    @classmethod
+    def is_integer_type(cls, value: str) -> bool:
+        try:
+            if value.startswith('0x'):
+                return value == hex(int(value, 16))
+        except ValueError:
+            return False
+        else:
+            return False
+
+    @classmethod
+    def is_address_type(cls, value: str) -> bool:
+        return is_icon_address_valid(value)
+
     def execute(self, context: 'IconScoreContext', params: dict, step_price: int, minimum_step: int):
         """Validate a transaction on icx_sendTransaction
         If failed to validate a tx, raise an exception
@@ -190,11 +294,13 @@ class IconPreValidator:
 
         # Check data_type-specific elements
         data_type = params.get('dataType', None)
-        if data_type == 'call':
+        self.validate_data_type(context, to, data_type)
+
+        if data_type == DataType.CALL:
             self._validate_call_transaction(context, params)
-        elif data_type == 'deploy':
+        elif data_type == DataType.DEPLOY:
             self._validate_deploy_transaction(context, params)
-        elif data_type == 'deposit':
+        elif data_type == DataType.DEPOSIT:
             self._validate_deposit_transaction(context, params)
 
     @staticmethod
@@ -220,6 +326,20 @@ class IconPreValidator:
             # it works like `call`.(calling fallback)
             context.engine.fee.check_score_available(context, to, context.block.height)
 
+    @staticmethod
+    def validate_data_type(
+            context: 'IconScoreContext', to: 'Address', data_type: Optional[str]):
+        if context.revision < Revision.IMPROVED_PRE_VALIDATOR.value:
+            return
+
+        if not DataType.contains(data_type):
+            raise InvalidParamsException(f"Invalid dataType: {data_type}")
+
+        if to.prefix == AddressPrefix.EOA and data_type not in (None, DataType.MESSAGE):
+            raise InvalidParamsException(
+                f"Mismatch between to and dataType: to={to}, dataType={data_type}"
+            )
+
     def _validate_call_transaction(self, context: 'IconScoreContext', params: dict):
         """Validate call transaction
         It is not icx_call
@@ -320,13 +440,10 @@ class IconPreValidator:
     def _check_balance(cls, context: 'IconScoreContext', from_: 'Address', value: int, fee: int):
         balance = context.engine.icx.get_balance(context, from_)
 
-        if is_address_locked(from_) and (
-                context.type == IconScoreContextType.QUERY or
-                (context.type == IconScoreContextType.INVOKE and
-                 context.revision == Revision.LOCK_ADDRESS.value)):
+        if context.revision >= Revision.LOCK_ADDRESS.value and is_address_locked(from_):
             Logger.warning(
                 tag="LOCK",
-                msg=f"Address is locked: balance={balance} from={str(from_)} value={value} fee={fee}"
+                msg=f"Address is locked: balance={balance} from={from_} value={value} fee={fee}"
             )
             raise InvalidRequestException(f"Address is locked: {from_}")
 
@@ -338,11 +455,11 @@ class IconPreValidator:
             raise OutOfBalanceException(msg)
 
     def _is_inactive_score(self, context: 'IconScoreContext', address: 'Address') -> bool:
-        is_contract = address.is_contract
-        is_zero_score_address = address == SYSTEM_SCORE_ADDRESS
-        is_score_active = self._is_score_active(context, address)
-        _is_inactive_score = is_contract and not is_zero_score_address and not is_score_active
-        return _is_inactive_score
+        return (
+            address.is_contract
+            and address != SYSTEM_SCORE_ADDRESS
+            and not self._is_score_active(context, address)
+        )
 
     @classmethod
     def _is_score_active(cls, context: 'IconScoreContext', address: 'Address') -> bool:
